/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "cube.h"

Cube::Cube( QWidget *parent, const char *name )
        : QGLWidget( parent, name )
//  : QGLWidget( QGLFormat( SingleBuffer|NoDepthBuffer ), parent, name )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	setFocusPolicy(QWidget::StrongFocus);

	timer = new QTimer( this, "mytimer" );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	alfa = beta = 0.0;

	initializeCube( );
	createCube( 0.6, cubo1 );
}


void Cube::timerDone( )
{
	alfa += 10.0*signo;
	beta += 5.0*signo;

	if ( alfa >= 360.0 ) alfa = 0.0;
	if ( alfa <= -360.0 ) alfa = 0.0;
	
	if ( beta >= 360.0 ) beta = 0.0;
	if ( beta <= -360.0 ) beta = 0.0;

	updateGL( );
}

void Cube::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
    xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Cube::keyPressEvent ( QKeyEvent * e )
{
	switch ( tolower(e->ascii()) ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Cube::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	/* initialize viewing values  */
	gluPerspective( 28.0, aspect, 1, 20 );
	
	emit changeSize ( );
}

QSizePolicy Cube::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Cube::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();             /* clear the matrix */

	glTranslatef( -0.3, -0.3, -3.3 );

	glRotatef (yrot, 0.0, 1.0, 0.0);
	glRotatef (xrot, 1.0, 0.0, 0.0);

	glRotatef ( 45.0, 0.0, 1.0, 0.0);
	glRotatef (-35.26, 1.0, 0.0, 0.0);
	glRotatef (alfa, 0.0, 0.0, 1.0);
	glRotatef ( 35.26, 1.0, 0.0, 0.0);
	glRotatef (-45.0, 0.0, 1.0, 0.0);

	/* viewing transformation  */

	showCube( cubo1 );

	swapBuffers( );
}

void Cube::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	glEnable (GL_DEPTH_TEST);


}

void Cube::initializeCube( )
{
	// La cara 1:
	cara[0][0]=2; cara[0][1]=6; cara[0][2]=7; cara[0][3]=3;
	color[0][0] = 1.0; color[0][1] = 1.0;  color[0][2] = 1.0; 

	// La cara 2: 1562
	cara[1][0]=0; cara[1][1]=4; cara[1][2]=5; cara[1][3]=1;
	color[1][0] = 0.0; color[1][1] = 1.0;  color[1][2] = 1.0; 

	// La cara 3: 5876
	cara[2][0]=4; cara[2][1]=7; cara[2][2]=6; cara[2][3]=5;
	color[2][0] = 1.0; color[2][1] = 0.0;  color[2][2] = 0.0; 

	// La cara 4: 1234
	cara[3][0]=0; cara[3][1]=1; cara[3][2]=2; cara[3][3]=3;
	color[3][0] = 0.0; color[3][1] = 0.0;  color[3][2] = 1.0; 

	// La cara 5: 2673
	cara[4][0]=1; cara[4][1]=5; cara[4][2]=6; cara[4][3]=2;
	color[4][0] = 0.0; color[4][1] = 1.0;  color[4][2] = 0.0; 

	// La cara 6: 1485
	cara[5][0]=0; cara[5][1]=3; cara[5][2]=7; cara[5][3]=4;
	color[5][0] = 1.0; color[5][1] = 1.0;  color[5][2] = 0.0; 
}

void Cube::createCube( float a, float v[][3] )
{
	v[0][0] = 0.0; v[0][1] = 0.0; v[0][2] = 0.0;
	v[1][0] = a;   v[1][1] = 0.0; v[1][2] = 0.0;
	v[2][0] = a;   v[2][1] = 0.0; v[2][2] = a;
	v[3][0] = 0.0; v[3][1] = 0.0; v[3][2] = a;
	v[4][0] = 0.0; v[4][1] = a;   v[4][2] = 0.0;
	v[5][0] = a;   v[5][1] = a;   v[5][2] = 0.0;
	v[6][0] = a;   v[6][1] = a;   v[6][2] = a;
	v[7][0] = 0.0; v[7][1] = a;   v[7][2] = a;
}

void Cube::showCube( float v[][3] )
{
	int i;

	for( i=0; i<6; i++ ) {
		glBegin(GL_QUADS);
		glColor3fv ( &color[i][0] );	
		glVertex3fv ( &v[ cara[i][0] ][0] );	
		glVertex3fv ( &v[ cara[i][1] ][0] );	
		glVertex3fv ( &v[ cara[i][2] ][0] );	
		glVertex3fv ( &v[ cara[i][3] ][0] );	
		glEnd();
	}
}
